@@ -17,9 +17,10 @@ class SpotifySessionManager(object):
     appkey_file = 'spotify_appkey.key'
     user_agent = 'pyspotify-example'
 
-    def __init__(self, username, password):
+    def __init__(self, username=None, password=None, remember_me=False):
         self.username = username
         self.password = password
+        self.remember_me = remember_me
         if self.application_key is None:
             self.application_key = open(self.appkey_file).read()
         self.awoken = threading.Event() # used to block until awoken
